/*****************************************************************************
 *   Copyright (C) 2009-2010 Peter "woggle" Mack, mac@denich.net             *
 *                                                                           *
 *   This program is free software; you can redistribute it and/or modify    *
 *   it under the terms of the GNU General Public License as published by    *
 *   the Free Software Foundation; either version 2 of the License.          *
 *                                                                           *
 *   This program is distributed in the hope that it will be useful,         *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of          *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the           *
 *   GNU General Public License for more details.                            *
 *                                                                           *
 *   You should have received a copy of the GNU General Public License       *
 *   along with this program; if not, write to the                           *
 *   Free Software Foundation, Inc.,                                         *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.               *
 *                                                                           *
 *****************************************************************************/




#include <avr/io.h>
#include <inttypes.h>
#include <stdlib.h>
#include <avr/pgmspace.h>
#include <avr/wdt.h>
#include <util/delay.h>
#include "main.h"
#include "lcd.h"
#include "parameter.h"

#include "menu.h"
#include "display.h"
#include "eeprom.h"


//#if defined HWVERSION3_1 || defined HWVERSION1_3
//#include "motortestI2C.h"
//#endif


#include "debug.h"
#include "timer.h"
#include "osd.h"
#include "motortest.h"
#include "gps.h"
//#include "pwm.h"
#include "eeprom.h"
#include "setup.h"
#include "uart1.h"
//#include "jeti.h"
#include "mk-data-structs.h"
#include "Wi232.h"
#include "servo.h"




#define ITEMS_NC 11

prog_char param_menuitems_nc[ITEMS_NC][17]= // zeilen,zeichen+1
{
		"OSD            ",
		"3D Lage        ",
		"Display        ",
		"Parameters     ",
		"Debug Data     ",
		"GPS Info       ",
		"Motor Tester   ",
		"Servo Tester   ",
		"PC BT  > Kopter",
		"PC USB > Kopter",
		"PKT Setup      ",

};

#define ITEMS_FC 8

prog_char param_menuitems_fc[ITEMS_FC][17]= // zeilen,zeichen+1
{
		"Display        ",
		"Parameters     ",
		"Debug Data     ",
		"Motor Tester   ",
		"Servo Tester   ",
		"PC BT  > Kopter",
		"PC USB > Kopter",
		"PKT Setup      ",
};

#ifdef HWVERSION3_9

#define ITEMS_NO 5

prog_char param_menuitems_no[ITEMS_NO][17]= // zeilen,zeichen+1
{
		"BL-Ctrl. Tester",
		"Servo Tester   ",
		"PC BT  > Kopter",
		"PC USB > Kopter",
		"PKT Setup      ",
};
#else
#define ITEMS_NO 2

prog_char param_menuitems_no[ITEMS_NO][15]= // zeilen,zeichen+1
{
	"BL-Ctrl. Test",
	"Setup PMK    "
};

#endif

//*****************************************************************************
// print cursor
void menu_set_cursor (uint8_t before, uint8_t line, uint8_t pos)
{
	lcd_printp_at (pos, before, PSTR(" "), 0);
	lcd_printp_at (pos, line, PSTR("\x1D"), 0);
}


//*****************************************************************************
// 
uint8_t menu_choose (uint8_t min, uint8_t max, uint8_t pos, uint8_t start)
{
	uint8_t line = start;
	uint8_t before = start;
	
	uint8_t k;

	menu_set_cursor (line, line, pos);
	
	do
	{
		if (get_key_press (1 << KEY_PLUS))
		{
			if (line < max)
			{
				line ++;
			}
			else
			{
				line = min;
			}
		}

		if (get_key_press (1 << KEY_MINUS))
		{
			if (line > min)
			{
				line --;
			}
			else
			{
				line = max;
			}
		}

		if (line != before)
		{
			menu_set_cursor (before, line, pos);
			before = line;
		}
	}
	while (!(k = get_key_press ((1 << KEY_ENTER) | (1 << KEY_ESC))));
	if (k & (1 << KEY_ESC))
	{
		line = 255;
	}

	return line;
}

uint8_t menu_choose2 (uint8_t min, uint8_t max,uint8_t start, uint8_t return_at_start,uint8_t return_at_end)
{
	uint8_t pos = 1;
	uint8_t line = start;
	uint8_t before = start;
	
	uint8_t k;

	menu_set_cursor (line, line, pos);
	
	do
	{
		if (get_key_press (1 << KEY_PLUS))
		{
			if (line < max)
			{
				line ++;
			}
			else
			{
				if(return_at_end == 1)
				{
					return 254;
				}
				else
				{
					//line = min;
				}
			}
		}

		if (get_key_press (1 << KEY_MINUS))
		{
			if (line > min)
			{
				line --;
			}
			else
			{
				if(return_at_start == 1)
				{
					return 253;
				}
				else
				{
					//line = max;
				}
			}
		}

		if (line != before)
		{
			menu_set_cursor (before, line, pos);
			before = line;
		}
	}
	while (!(k = get_key_press ((1 << KEY_ENTER) | (1 << KEY_ESC))));
	if (k & (1 << KEY_ESC))
	{
		line = 255;
	}

	return line;
}

uint8_t menu_choose3 (uint8_t min, uint8_t max,uint8_t start, uint8_t return_at_start,uint8_t return_at_end)
{
	uint8_t pos = 1;
	uint8_t line = start;
	uint8_t before = start;
	
	menu_set_cursor (line, line, pos);
	
	do
	{
		if (get_key_press (1 << KEY_PLUS))
		{
			if (line < max)
			{
				line ++;
			}
			else
			{
				if(return_at_end == 1)
				{
					return 254;
				}
				else
				{
					//line = min;
				}
			}
		}

		if (get_key_press (1 << KEY_MINUS))
		{
			if (line > min)
			{
				line --;
			}
			else
			{
				if(return_at_start == 1)
				{
					return 253;
				}
				else
				{
					//line = max;
				}
			}
		}
#ifdef HWVERSION3_9
		if (get_key_press (1 << KEY_ESC))
			{
			copy_line(7);
			lcd_printp_at (0, 7, PSTR("AUS?Taste festhalten"), 2);
			_delay_ms (400);	//Ausschaltverzögerung

			if (!(PINA & (1<<PINA5)))
				{
//				WriteLastPosition(0x10001000,0x20002000);   test Lastposition
			    beeptime = 250;
			    BeepMuster = 0x0080;
				_delay_ms(400);
				clr_V_On();			//;		Spannung abschalten
				}
			else
				paste_line(7);
			}
#endif

		if (line != before)
		{
			menu_set_cursor (before, line, pos);
			before = line;
		}

	}
	while (!(get_key_press (1 << KEY_ENTER)));

	return line;
}

void main_menu(void)
{
	uint8_t ii = 0;
	uint8_t offset = 0;
	uint8_t size = 0;


	uint8_t Save_hardware = 0;
	uint8_t dmode = 0;
	uint8_t target_pos = 1;
	uint8_t val =0;
	Save_hardware = hardware;

	while(1)
	{


	if (Debug==1) hardware = NC;
	if (Debug==0) hardware = Save_hardware;

	if(hardware == NC) size = ITEMS_NC ;
	if(hardware == FC) size = ITEMS_FC ;
	if(hardware == NO) size = ITEMS_NO ;

	lcd_cls ();
	lcd_printp_at (0, 0, PSTR("PKT-Tool=FC "), 2);
	lcd_printp (PSTR(FC_Version),2);
//	lcd_line(0,7,128,7,1);

	lcd_printpns_at (0, 7, PSTR(" \x1a    \x1b     off    \x0c"), 0);
	
	
	while(1)
	{	

		ii = 0;
		if(offset > 0)
		{
			lcd_printp_at(1,1, PSTR("\x1a"), 0);
		}
		for(ii = 0;ii < 6 ; ii++)
		{
			if((ii+offset) < size)
			{
				if(hardware == NC)

				{
					lcd_printp_at(3,ii+1,param_menuitems_nc[ii+offset], 0);
				}

				else

				if(hardware == FC)
				{
					lcd_printp_at(3,ii+1,param_menuitems_fc[ii+offset], 0);
				}


				else
				{
					lcd_printp_at(3,ii+1,param_menuitems_no[ii+offset], 0);
				}
			}
			if((ii == 5)&&(ii+offset < (size-1)))
			{
				lcd_printp_at(1,6, PSTR("\x1b"), 0);
			}
		
		}
		
		if(dmode == 0)
		{
			if(offset == 0)
			{
				if(size > 6)
				{
					val = menu_choose3 (1, 5, target_pos,0,1);
				}
				else
				{
					val = menu_choose3 (1, size, target_pos,0,0);
				}
			}
			else
			{
				val = menu_choose3 (2, 5, target_pos,1,1);
			}
		}


		if(dmode == 1)
		{
			if(offset+7 > size)
			{
				val = menu_choose3 (2, 6, target_pos,1,0);
			}
			else
			{
				val = menu_choose3 (2, 5, target_pos,1,1);
			}
		}



		if(val == 254)
		{
			offset++;
			dmode = 1;
			target_pos = 5;

		}else if(val == 253)
		{

			offset--;
			dmode = 0;
			target_pos = 2;
		}
		else
		if(val == 255)
		{
			//nothing
		}

		else
		{
			break;
		}
	}

	target_pos = val;



	if(hardware == NC)
	{
		if((val+offset) == 1 )  osd(OSD_Mode);
		if((val+offset) == 2 )  osd(THREE_D_Mode);
		if((val+offset) == 3 )  display_data();
		if((val+offset) == 4 )  edit_parameter();
		if((val+offset) == 5 ) display_debug();
		if((val+offset) == 6 ) gps();
		if((val+offset) == 7 ) motor_test(FC_Mode);
		if((val+offset) == 8 ) servo_test();

#if defined HWVERSION3_9
		if (U02SV2 == 0)
		{
			if((val+offset) == 9 ) Port_BT2Wi();
			if((val+offset) == 10) Port_USB2Wi();
		}
		else if (U02SV2 == 1)
		{
			if((val+offset) == 9 ) Port_BT2FC();
			if((val+offset) == 10) Port_USB2FC();
		}
#else
		if((val+offset) == 9)   Show_Error_HW12();
		if((val+offset) == 10)   Show_Error_HW12();

#endif
		if((val+offset) == 11) PKT_Setup();
	}



	if(hardware == FC)
	{
		if((val+offset) == 1 ) display_data();
		if((val+offset) == 2 ) edit_parameter();
		if((val+offset) == 3 ) display_debug();
		if((val+offset) == 4 ) motor_test(FC_Mode);
		if((val+offset) == 5 ) servo_test();

#if defined HWVERSION3_9
		if (U02SV2 == 0)
			{
				if((val+offset) == 6 ) Port_BT2Wi();
				if((val+offset) == 7 ) Port_USB2Wi();
			}
			else if (U02SV2 == 1)
			{
				if((val+offset) == 6 ) Port_BT2FC();
				if((val+offset) == 7 ) Port_USB2FC();
			}
#else
		if((val+offset) == 6)   Show_Error_HW12();
		if((val+offset) == 7)   Show_Error_HW12();
#endif
		if((val+offset) == 8 ) PKT_Setup();
	}



	if(hardware == NO)
	{
		if((val+offset) == 1 ) motor_test(FC_Mode);
		if((val+offset) == 2 ) servo_test();
		#if defined HWVERSION3_9
			if (U02SV2 == 0)
			{
				if((val+offset) == 3 ) Port_BT2Wi();
				if((val+offset) == 4 ) Port_USB2Wi();
			}
			else if (U02SV2 == 1)
			{
				if((val+offset) == 3 ) Port_BT2FC();
				if((val+offset) == 4 ) Port_USB2FC();
			}
		#else
			if((val+offset) == 3 ) Show_Error_HW12();
			if((val+offset) == 4 ) Show_Error_HW12();
		#endif
		if((val+offset) == 5 ) PKT_Setup();

	}
  }
}


void Update_PKT(void)
{


				lcd_cls();
				lcd_printpns_at (0, 0, PSTR("Connect PC to PKT-USB"),0);
				lcd_printpns_at (0, 1, PSTR("Press 'Start' on PKT"),0);
				lcd_printpns_at (0, 2, PSTR("Then start avrdude:"),0);
			    lcd_printpns_at (0, 3, PSTR("avrdude -pm644p -cavr"),0);
			    lcd_printpns_at (0, 4, PSTR("109 -Pcom? -b115200 -"),0);
			    lcd_printpns_at (0, 5, PSTR("Uflash:w:NEWSOFTWARE"),0);
			    lcd_printpns_at (0, 6, PSTR(".hex:a"),0);
	//					    avrdude -pm644p -cavr109 -P/dev/ttyUSB1 -b115200 -V -Uflash:w:GPL_PKT_V3.1L_FC0.84_HW3x.hex:a

#ifdef HaraldB
			    lcd_printpns_at (0, 7, PSTR("Start            Back"),0);

			    do

			    if ((get_key_press (1 << KEY_ENTER))) {return;}

			    while (!(get_key_press (1 << KEY_MINUS)));
				    {
						/* start bootloader with Reset, Hold KEY_ENTER*/
					 wdt_enable( WDTO_250MS );
					 while (1) { ; }
					}
#else
				    lcd_printpns_at (0, 7, PSTR("Back            Start"),0);

							    do

							    if ((get_key_press (1 << KEY_MINUS))) {return;}

							    while (!(get_key_press (1 << KEY_ENTER)));
								    {
										/* start bootloader with Reset, Hold KEY_ENTER*/
									 wdt_enable( WDTO_250MS );
									 while (1) { ; }
									}
#endif


}



void Test_Routine(void)
{
	lcd_cls ();
	lcd_printpns_at (0,2,PSTR("lösche WP Liste"), 0);
	EEWayPointList_Clear();
	lcd_printpns_at (0,3,PSTR("fertig"), 0);
	while (!(get_key_press (1 << KEY_ENTER)));
	return;
}


void Show_Version(void)
{
	lcd_cls ();
//	lcd_frect (0,1,127,7,1);
	lcd_printpns_at (0,0,PSTR("PMK Tool             "), 2);
	lcd_printpns_at (9,0,PSTR(PKTSWVersion),2);
	lcd_printpns_at (0,1,PSTR("for FC               "), 2);
	lcd_printpns_at (8,1,PSTR(FC_Version),2);
	lcd_printpns_at (0,2,PSTR("(C) GNU GPL License"), 0);
	lcd_printpns_at (0,3,PSTR("   NO WARRANTY"), 0);
	lcd_printpns_at (0,4,PSTR("2008 Thomas Kaiser"), 0);
	lcd_printpns_at (0,5,PSTR("2009-2010 Peter Mack"), 0);
	lcd_printpns_at (0,6,PSTR("2011 Chr. Brandtner "), 0);
//	lcd_printpns_at (0,7,PSTR("2010 Sebastian Boehm"), 0);


	while (!(get_key_press (1 << KEY_ENTER)));
	return;

}




